﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Markup;

namespace TestProject
{
    /// <summary>
    /// 메인 애플리케이션
    /// </summary>
    public partial class MainApplication : Application
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 스킨 - Skin

        /// <summary>
        /// 스킨
        /// </summary>
        public static Skin Skin { get; set; } = Skin.Red;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 스킨 변경하기 - ChangeSkin(Skin skin)

        /// <summary>
        /// 스킨 변경하기
        /// </summary>
        /// <param name="skin">스킨</param>
        public void ChangeSkin(Skin skin)
        {
            Skin = skin;

            Resources.Clear();

            Resources.MergedDictionaries.Clear();

            if(Skin == Skin.Blue)
            {
                AddResource("RESOURCE/BlueSkin.xaml");
            }
            else if(Skin == Skin.Green)
            {
                AddLooseReource("RESOURCE/GreenSkin.xaml");
            }
            else if(Skin == Skin.Red)
            {
                AddResource("RESOURCE/RedSkin.xaml");
            }

            AddResource("RESOURCE/SharedStyle.xaml"); 
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 시작시 처리하기 - OnStartup(e)

        /// <summary>
        /// 시작시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnStartup(StartupEventArgs e)
        {
            base.OnStartup(e);

            ChangeSkin(Skin.Red);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 리소스 추가하기 - AddResource(uri)

        /// <summary>
        /// 리소스 추가하기
        /// </summary>
        /// <param name="uri">URI</param>
        private void AddResource(string uri)
        {
            ResourceDictionary dictionary = new ResourceDictionary()
            {
                Source = new Uri(uri, UriKind.Relative)
            };

            foreach(ResourceDictionary mergedDictionary in dictionary.MergedDictionaries)
            {
                Resources.MergedDictionaries.Add(mergedDictionary);
            }
 
            foreach(object key in dictionary.Keys)
            {
                Resources[key] = dictionary[key];
            }
        }

        #endregion
        #region 느슨한 리소스 추가하기 - AddLooseReource(filePath)

        /// <summary>
        /// 느슨한 리소스 추가하기
        /// </summary>
        /// <param name="filePath">파일 경로</param>
        private void AddLooseReource(string filePath)
        {
            StreamReader reader = new StreamReader(filePath);

            ResourceDictionary dictionary = XamlReader.Load(reader.BaseStream) as ResourceDictionary;

            Resources.MergedDictionaries.Add(dictionary);
        }

        #endregion
    }
}